// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// This Source Code Form is subject to the terms of the X11 (MIT) License.
// If a copy of the X11 License was not distributed with this file, 
// You can obtain one at https://opensource.org/licenses/MIT/.
//
// X11 (MIT) License
// 
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef VKF_BEGET_HPP
#define VKF_BEGET_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <vector>
#include <cassert>
#include <cmath>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

//#include "Join.hpp"

namespace vkf {

class Beget : public std::map<std::pair<std::string,std::string>, std::pair<double,std::pair<double,double>>> {
  public:
    Beget(  const std::string &complex_table_name,
            const std::string &database,
            const std::string &db_host = DEFAULT_HOST,
            const std::string &db_user = DEFAULT_USER,
            const std::string &db_pswd = DEFAULT_PASS
        ) {
        // set up the account
        mariadb::account_ref acc = mariadb::account::create(db_host, db_user, db_pswd);
        // create connection
        mariadb::connection_ref con = mariadb::connection::create(acc);
        con->execute("USE " + database);
        // obtain content of table encoder
        mariadb::result_set_ref result = con->query("SELECT * FROM " + complex_table_name);
        con->~connection();
        acc->~account();
        // reading from a result set
        while (result->next()) {
            this->emplace(std::make_pair(result->get_string("first_name"),result->get_string("second_name")),
                std::make_pair(result->get_double("bias"),
                    std::make_pair(result->get_double("first_coeff"), result->get_double("first_coeff"))));
        }
    };

    ~Beget() = default;

    double Estimate(double value1, double value2, const std::string name1, const std::string name2) const {
        std::pair<double,std::pair<double,double>> weights = this->find(std::pair<std::string,std::string>(name1,name2))->second;
        return weights.first + weights.second.first*value1 + weights.second.second*value2;
    };

};

} // namespace vkf

#endif // VKF_BEGET_HPP

